package ovh.gasser.newshapes.ui;

import io.reactivex.disposables.Disposable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import ovh.gasser.newshapes.Selection;
import ovh.gasser.newshapes.util.EventSource;
import ovh.gasser.newshapes.shapes.Shape;
import ovh.gasser.newshapes.shapes.SCollection;

import java.awt.event.MouseEvent;
import java.util.Optional;

public class Controller {
    private final Logger logger = LoggerFactory.getLogger(ShapesView.class);
    private final Disposable mouseSub;
    private Selection selection;

    Controller(ShapesView view, Shape model) {
        mouseSub = EventSource
                .fromMouseEventsOf(view)
                .subscribe(evt -> {
                    assert model instanceof SCollection;
                    SCollection sc = (SCollection) model;
                    switch (evt.getID()) {
                        case MouseEvent.MOUSE_PRESSED:
                            getTarget(evt, sc)
                                    .ifPresentOrElse(
                                            s -> selection = new Selection(s, true),
                                            () -> {
                                                if (selection != null) {
                                                    selection.unselect();
                                                    selection = null;
                                                }
                                            }
                                    );
                            break;
                        case MouseEvent.MOUSE_DRAGGED:
                            handleMouseMoved(evt);
                            break;
                    }
                    view.repaint();
                    }, err -> logger.error("{}", err)
                );
    }

    private Optional<Shape> getTarget(MouseEvent evt, SCollection sc) {
        return sc.stream()
                .filter(s -> s.getBounds().contains(evt.getPoint()))
                .findFirst();
    }

    private void handleMouseMoved(MouseEvent evt) {
        if (selection != null) selection.shape.setLoc(evt.getPoint());
    }

    public void dispose() {
        logger.info("Cleaning subscriptions...");
        mouseSub.dispose();
    }
}
